package com.dky.security;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.SM2;
import com.dky.sm2.SM4;
import com.dky.sm2.SM4_Context;
import com.dky.sm2.Util;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Security;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SM4Utils {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    // public final static String SM4_KEY = "1100fba8ee67ddf1f6f4e37c500dc10eee1bf15827ae3837810e30f402fa0bc6";

    public final static String SM4_KEY = "dfd2b0581fc9244e51100c5cf02defc8064a45686eb2e4ddfd44cdab2ef908b2";

    private static final String ALGORITHM = "AES/ECB/PKCS5Padding";

    private static final String UTF_8 = "UTF-8";

    private static final boolean HEX_STRING = false;

    public static String encrypt(Map<String, String> map) throws Exception {

        Cipher cipher = Cipher.getInstance(ALGORITHM, "BC");
        SecretKeySpec secretKey = new SecretKeySpec(SM4_KEY.getBytes(StandardCharsets.UTF_8), "AES");
        cipher.init(Cipher.ENCRYPT_MODE, secretKey);
        byte[] encrypted = cipher.doFinal(mapToString(map).getBytes(StandardCharsets.UTF_8));
        return Base64.getEncoder().encodeToString(encrypted);
    }


    private static String mapToString(Map<String, String> map) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            sb.append(entry.getKey()).append('=').append(entry.getValue()).append('&');
        }
        return sb.toString();
    }

    private static Map<String, String> stringToMap(String str) {
        Map<String, String> map = new HashMap<>();
        String[] entries = str.split("&");
        for (String entry : entries) {
            String[] keyValue = entry.split("=", 2);
            if (keyValue.length == 2) {
                map.put(keyValue[0], keyValue[1]);
            }
        }
        return map;
    }


    /**
     * SM2私钥解密
     *
     * @param encryptStr SM2加密字符串
     * @return
     */
    public static String sm2DecryptBase64(String encryptStr) {
        try {
            SM2 sm2 = new SM2(SM4_KEY, null);
            return StrUtil.utf8Str(sm2.decrypt(encryptStr, KeyType.PrivateKey));
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("密钥错误，请检查密钥输入是否正确");
        }
        return null;
    }

    // ECB模式加密
    public static String encryptData_ECB(String plainText) {
        try {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_ENCRYPT;

            byte[] keyBytes;
            keyBytes = SM4_KEY.getBytes();
            SM4 sm4 = new SM4();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            byte[] encrypted = sm4.sm4_crypt_ecb(ctx, plainText.getBytes(UTF_8));
            String cipherText = new BASE64Encoder().encode(encrypted);
            if (cipherText != null && cipherText.trim().length() > 0) {
                Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                Matcher m = p.matcher(cipherText);
                cipherText = m.replaceAll("");
            }
            return cipherText;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // ECB模式解密
    public static String decryptData_ECB(String cipherText) {
        try {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_DECRYPT;

            byte[] keyBytes;
            keyBytes = SM4_KEY.getBytes();
            SM4 sm4 = new SM4();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            byte[] decrypted = sm4.sm4_crypt_ecb(ctx, new BASE64Decoder().decodeBuffer(cipherText));
            return new String(decrypted, UTF_8);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // CBC模式加密
    public static String encryptData_CBC(String plainText) {
        try {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_ENCRYPT;

            byte[] keyBytes;
            byte[] ivBytes;

            keyBytes = SM4_KEY.getBytes();
            ivBytes = ALGORITHM.getBytes();

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            byte[] encrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, plainText.getBytes(UTF_8));
            String cipherText = new BASE64Encoder().encode(encrypted);
            if (cipherText != null && cipherText.trim().length() > 0) {
                Pattern p = Pattern.compile("\\s*|\t|\r|\n");
                Matcher m = p.matcher(cipherText);
                cipherText = m.replaceAll("");
            }
            return cipherText;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    // CBC模式解密
    public static String decryptData_CBC(String cipherText) {
        try {
            SM4_Context ctx = new SM4_Context();
            ctx.isPadding = true;
            ctx.mode = SM4.SM4_DECRYPT;

            byte[] keyBytes;
            byte[] ivBytes;
            if (HEX_STRING) {
                keyBytes = Util.hexStringToBytes(SM4_KEY);
                ivBytes = Util.hexStringToBytes(ALGORITHM);
            } else {
                keyBytes = SM4_KEY.getBytes();
                ivBytes = ALGORITHM.getBytes();
            }

            SM4 sm4 = new SM4();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            byte[] decrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, new BASE64Decoder().decodeBuffer(cipherText));
            return new String(decrypted, UTF_8);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private static String mapTostring(Map<String, Object> map) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            sb.append(entry.getKey()).append('=').append(entry.getValue()).append('&');
        }
        return sb.toString();
    }

    public static void main(String[] args) {

        Map<String, Object> map = new HashMap<>();
        map.put("cpuIds", "2024-03-30 19:00:42");
        map.put("expireTime", "2025-03-30 19:00:42");
        map.put("companyName", "中能睿通");

        String cipherText = SM4Utils.encryptData_ECB(mapTostring(map));
        System.out.println("ECB模式加密密文: " + cipherText + "\n");

    }

}